package gov.cms.fiss.pricers.irf.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.api.ProviderRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import gov.cms.fiss.pricers.irf.core.ResultCode;
import gov.cms.fiss.pricers.irf.core.tables.CbsaTableEntry;
import java.math.BigDecimal;
import java.math.RoundingMode;
import org.apache.commons.lang3.StringUtils;

public class ApplySupplementalWageIndex
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public boolean shouldExecute(IrfPricerContext calculationContext) {
    return calculationContext.matchesReturnCode(ResultCode.OK_00)
        && StringUtils.equals(
            calculationContext.getProviderRecord().getSupplementalWageIndexIndicator(), "1");
  }

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProviderRecord providerRecord = calculationContext.getProviderRecord();
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // Check for presence of supplemental wage index value
    if (null == providerRecord.getSupplementalWageIndex()
        || BigDecimalUtils.isZero(providerRecord.getSupplementalWageIndex())) {
      calculationContext.applyResultCode(
          ResultCode.INVALID_WAGE_INDEX_52, "Supplemental wage index required");

      return;
    }

    //  Verify that supplemental wage index is in fiscal year
    if (!LocalDateUtils.inRange(
        calculationContext.getProviderEffectiveDate(),
        calculationContext.getFiscalYearStart(),
        calculationContext.getFiscalYearEnd())) {
      calculationContext.applyResultCode(
          ResultCode.INVALID_WAGE_INDEX_52, "Supplemental wage index not within fiscal year");

      return;
    }

    final CbsaTableEntry cbsaWageIndex = calculationContext.getCbsaWageIndex();
    if (null != cbsaWageIndex.getEffectiveDate()) {
      capWageIndexDecrease(cbsaWageIndex, calculationContext);
    }

    // Update output record with CBSA data
    prospectivePaymentRecord.setCoreBasedStatisticalArea(
        calculationContext.getCbsaWageIndex().getCbsa());
    prospectivePaymentRecord.setWageIndex(
        calculationContext.getCbsaWageIndex().getWageIndex().setScale(4, RoundingMode.HALF_UP));
  }

  /**
   * Applies a reduction cap to the supplemental wage index if the difference between CBSA and
   * supplemental * wage indexes is less than the reduction cap percentage.
   */
  private void capWageIndexDecrease(
      CbsaTableEntry cbsaTableEntry, IrfPricerContext calculationContext) {
    final ProviderRecord providerRecord = calculationContext.getProviderRecord();

    final BigDecimal wageIndexPercentReduction =
        cbsaTableEntry
            .getWageIndex()
            .subtract(providerRecord.getSupplementalWageIndex())
            .divide(providerRecord.getSupplementalWageIndex(), RoundingMode.HALF_UP);

    if (BigDecimalUtils.isLessThan(
        wageIndexPercentReduction, calculationContext.getWageIndexPercentReductionCap())) {
      cbsaTableEntry.setWageIndex(
          providerRecord
              .getSupplementalWageIndex()
              .multiply(calculationContext.getWageIndexPercentAdjustment()));
    }
  }
}
