package gov.cms.fiss.pricers.irf.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.irf.api.BillingRecord;
import gov.cms.fiss.pricers.irf.api.CbsaData;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.api.ProviderRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import gov.cms.fiss.pricers.irf.core.ResultCode;
import gov.cms.fiss.pricers.irf.core.tables.CbsaTableEntry;
import java.math.RoundingMode;
import org.apache.commons.lang3.StringUtils;

/**
 * Determines the wage index.
 *
 * <p>Converted from {@code 0550-GET-CBSA} and {@code 0650-N-GET-WAGE-INDX} in the DRV COBOL code.
 */
public class DefaultWageIndexRetrieval
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // Set wage index
    determineWageIndex(calculationContext);
    // AT END
    if (calculationContext.getCbsaWageIndex() == null) {
      // MOVE 60 TO PPS-RTC
      calculationContext.applyResultCode(ResultCode.WAGE_INDEX_NOT_FD_60);
      return;
    }

    // Update output record with CBSA data
    prospectivePaymentRecord.setCoreBasedStatisticalArea(
        calculationContext.getCbsaWageIndex().getCbsa());
    prospectivePaymentRecord.setWageIndex(
        calculationContext.getCbsaWageIndex().getWageIndex().setScale(4, RoundingMode.HALF_UP));
  }

  /**
   * Determines the source of the wage index information (provider vs. CBSA table) and updates the
   * context accordingly.
   *
   * @param calculationContext the current execution context
   */
  private void determineWageIndex(IrfPricerContext calculationContext) {
    final BillingRecord billingRecord = calculationContext.getBillingRecord();
    final ProviderRecord providerRecord = calculationContext.getProviderRecord();
    final CbsaData cbsa = providerRecord.getCbsaData();

    // If the special payment indicator is 1 then create a CBSA record based on provider record
    // values otherwise retrieve the CBSA record from the lookup table.

    // IF P-NEW-CBSA-SPEC-PAY-IND = '1'
    if (StringUtils.equals(
        cbsa.getSpecialPaymentIndicator(), IrfPricerContext.SPECIAL_PAY_INDICATOR)) {
      // MOVE P-NEW-CBSA-WAGE-INDEX TO
      //      W-NEW-INDEX-RECORD-C
      // MOVE P-NEW-GEO-LOC-CBSA9 TO W-NEW-CBSA
      calculationContext.setCbsaWageIndex(
          new CbsaTableEntry(cbsa.getGeolocation(), null, cbsa.getWageIndex()));
    } else {
      // ELSE
      //    SEARCH M-CBSA-DATA
      calculationContext.setCbsaWageIndex(
          calculationContext.getCbsaWageIndex(
              cbsa.getGeolocation(), billingRecord.getDischargeDate()));
    }
  }
}
