package gov.cms.fiss.pricers.irf.core.rules.finalize_payment;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * Calculates payment amounts.
 *
 * <p>Converted from {@code 5000-FINAL-PAYMENTS} in the COBOL code.
 */
// TODO: DDS to evaluate for a better name - these are adjustments
public class CalculatePaymentAmounts
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    // ** RULE: CLAIMS WHERE THE FED IS PAYING 100% OF THE PAYMENT ARE
    // ** ASSIGNED ZERO DOLLARS FOR THE PPS FACILITY SPECIFIC PAYMENT
    // ** AMOUNT
    // ** RULE: CLAIMS WHERE THE FEDS ARE NOT PAYING 100% OF THE BILL
    // ** HAVE THEIR PPS FEDERAL PAYMENT AMOUNT COMPUTED AS THE (PPS
    // ** FEDERAL RATE PERCENT TIMES THE PPS FEDERAL PAYMENT AMOUNT)
    // ** ROUNDED

    // IF PPS-FED-RATE-PCT = 1.0000
    if (BigDecimalUtils.equals(BigDecimal.ONE, prospectivePaymentRecord.getFederalRatePercent())) {
      // MOVE 0 TO PPS-FAC-SPEC-PAY-AMT
      prospectivePaymentRecord.setFacilitySpecificPaymentAmount(BigDecimalUtils.ZERO);
    } else {
      // ** RULE: CLAIMS WHERE THE FEDS ARE NOT PAYING 100% OF THE BILL
      // ** HAVE THEIR PPS FACILITY SPECIFIC PAYMENT AMOUNT COMPUTED AS
      // ** THE PPS FACILITY SPECIFIC RATE PERCENT TIMES THE PPS FACILITY
      // ** SPECIFIC RATE PREBLEND AMOUNT) ROUNDED

      // ELSE
      //    COMPUTE PPS-FED-PAY-AMT ROUNDED =
      //            (PPS-FED-RATE-PCT * PPS-FED-PAY-AMT)
      prospectivePaymentRecord.setFederalPaymentAmount(
          prospectivePaymentRecord
              .getFederalRatePercent()
              .multiply(prospectivePaymentRecord.getFederalPaymentAmount())
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: CLAIMS WHERE THE FEDS ARE NOT PAYING 100% OF THE BILL
      // ** HAVE THEIR PPS OUTLIER PAYMENT AMOUNT COMPUTED AS THE (PPS
      // ** FEDERAL RATE PERCENT TIMES THE PPS OUTLIER PAYMENT AMOUNT)
      // ** ROUNDED

      //    COMPUTE PPS-FAC-SPEC-PAY-AMT ROUNDED =
      //            (PPS-FAC-RATE-PCT * PPS-FAC-SPEC-RT-PREBLEND)
      prospectivePaymentRecord.setFacilitySpecificPaymentAmount(
          prospectivePaymentRecord
              .getFacilityRatePercent()
              .multiply(prospectivePaymentRecord.getFacilitySpecificRatePreblend())
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: CLAIMS WHERE THE FEDS ARE NOT PAYING 100% OF THE BILL
      // ** HAVE THEIR PPS TEACH PAYMENT AMOUNT COMPUTED AS THE (PPS
      // ** FEDERAL RATE PERCENT TIMES THE PPS TEACH PAYMENT AMOUNT)
      // ** ROUNDED

      //    COMPUTE PPS-OUTLIER-PAY-AMT ROUNDED =
      //            (PPS-FED-RATE-PCT * PPS-OUTLIER-PAY-AMT)
      prospectivePaymentRecord.setOutlierPaymentAmount(
          prospectivePaymentRecord
              .getFederalRatePercent()
              .multiply(prospectivePaymentRecord.getOutlierPaymentAmount())
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: CLAIMS WHERE THE FEDS ARE NOT PAYING 100% OF THE BILL
      // ** HAVE THEIR PPS LOW INCOME PATIENT PAYMENT AMOUNT COMPUTED AS
      // ** THE (PPS FEDERAL RATE PERCENT TIMES THE PPS LOW INCOME PATIENT
      // ** PAYMENT AMOUNT) ROUNDED

      //    COMPUTE PPS-TEACH-PAY-AMT ROUNDED =
      //            (PPS-FED-RATE-PCT * PPS-TEACH-PAY-AMT)
      prospectivePaymentRecord.setTeachPaymentAmount(
          prospectivePaymentRecord
              .getFederalRatePercent()
              .multiply(prospectivePaymentRecord.getTeachPaymentAmount())
              .setScale(2, RoundingMode.HALF_UP));

      //    COMPUTE PPS-LIP-PAY-AMT ROUNDED =
      //            (PPS-FED-RATE-PCT * PPS-LIP-PAY-AMT).
      prospectivePaymentRecord.setLowIncomePaymentAmount(
          prospectivePaymentRecord
              .getFederalRatePercent()
              .multiply(prospectivePaymentRecord.getLowIncomePaymentAmount())
              .setScale(2, RoundingMode.HALF_UP));
    }
  }
}
