package gov.cms.fiss.pricers.irf.core.rules.finalize_payment;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import java.math.RoundingMode;

/**
 * Calculates penalty amounts.
 *
 * <p>Converted from {@code 5000-FINAL-PAYMENTS} in the COBOL code.
 */
public class CalculatePenaltyAmounts
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS FEDERAL PENALTY
    // ** AMOUNT COMPUTED AS THE (PPS FEDERAL PAY AMOUNT TIMES 25%)
    // ** ROUNDED. IN OTHER WORDS, THEY PAY A 25% PENALTY OF THE FEDERAL
    // ** PAID AMOUNT

    // IF B-SPEC-PAY-IND = '2' OR '3'
    if (calculationContext.hasPenalties()) {
      // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS FEDERAL PENALTY
      // ** AMOUNT COMPUTED AS THE (PPS FEDERAL PAY AMOUNT TIMES 25%)
      // ** ROUNDED. THE PENALTY AMOUNT IS SUBTRACTED FROM THE FEDERAL
      // ** PAY AMOUNT

      // COMPUTE PPS-FED-PENALTY-AMT ROUNDED =
      //           (PPS-FED-PAY-AMT * .25)
      prospectivePaymentRecord.setFederalPenaltyAmount(
          prospectivePaymentRecord
              .getFederalPaymentAmount()
              .multiply(IrfPricerContext.TWENTY_FIVE_PERCENT)
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS FEDERAL LOW
      // ** INCOME PATIENT PENALTY AMOUNT COMPUTED AS THE (PPS LOW INCOME
      // ** PATIENT PAY AMOUNT TIMES 25%) ROUNDED. IN OTHER WORDS, THEY
      // ** PAY A 25% LOW INCOME PATIENT PENALTY

      // COMPUTE PPS-FED-PAY-AMT =
      //            (PPS-FED-PAY-AMT - PPS-FED-PENALTY-AMT)
      prospectivePaymentRecord.setFederalPaymentAmount(
          prospectivePaymentRecord
              .getFederalPaymentAmount()
              .subtract(prospectivePaymentRecord.getFederalPenaltyAmount())
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS LOW INCOME
      // ** PATIENT PENALTY AMOUNT COMPUTED AS THE (PPS LOW INCOME PATIENT
      // ** PAY AMOUNT MINUS THE PPS LOW INCOME PATIENT PENALTY AMOUNT)
      // ** ROUNDED.

      // COMPUTE PPS-LIP-PENALTY-AMT ROUNDED =
      //            (PPS-LIP-PAY-AMT * .25)
      prospectivePaymentRecord.setLowIncomePaymentPenaltyAmount(
          prospectivePaymentRecord
              .getLowIncomePaymentAmount()
              .multiply(IrfPricerContext.TWENTY_FIVE_PERCENT)
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS OUTLIER PENALTY
      // ** AMOUNT COMPUTED AS THE (PPS OUTLIER PAY AMOUNT TIMES 25%)
      // ** ROUNDED. IN OTHER WORDS, THEY PAY A 25% OUTLIER PENALTY

      // COMPUTE PPS-LIP-PAY-AMT =
      //            (PPS-LIP-PAY-AMT - PPS-LIP-PENALTY-AMT)
      prospectivePaymentRecord.setLowIncomePaymentAmount(
          prospectivePaymentRecord
              .getLowIncomePaymentAmount()
              .subtract(prospectivePaymentRecord.getLowIncomePaymentPenaltyAmount())
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS OUTLIER PENALTY
      // ** AMOUNT COMPUTED AS THE (PPS OUTLIER PAY AMOUNT MINUS THE PPS
      // ** OUTLIER PENALTY AMOUNT) ROUNDED.

      // COMPUTE PPS-OUT-PENALTY-AMT ROUNDED =
      //            (PPS-OUTLIER-PAY-AMT * .25)
      prospectivePaymentRecord.setOutlierPenaltyAmount(
          prospectivePaymentRecord
              .getOutlierPaymentAmount()
              .multiply(IrfPricerContext.TWENTY_FIVE_PERCENT)
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS TEACH PENALTY
      // ** AMOUNT COMPUTED AS THE (PPS TEACH PAY AMOUNT TIMES 25%)
      // ** ROUNDED. IN OTHER WORDS, THEY PAY A 25% PENALTY OF THE TEACH
      // ** PAID AMOUNT

      // COMPUTE PPS-OUTLIER-PAY-AMT =
      //            (PPS-OUTLIER-PAY-AMT - PPS-OUT-PENALTY-AMT)
      prospectivePaymentRecord.setOutlierPaymentAmount(
          prospectivePaymentRecord
              .getOutlierPaymentAmount()
              .subtract(prospectivePaymentRecord.getOutlierPenaltyAmount())
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS TEACH PENALTY
      // ** AMOUNT COMPUTED AS THE (PPS TEACH PAY AMOUNT MINUS THE PPS THE
      // ** PPS TEACH PENALTY AMOUNT) ROUNDED.

      // COMPUTE PPS-TEACH-PENALTY-AMT ROUNDED =
      //            (PPS-TEACH-PAY-AMT * .25)
      prospectivePaymentRecord.setTeachPenaltyAmount(
          prospectivePaymentRecord
              .getTeachPaymentAmount()
              .multiply(IrfPricerContext.TWENTY_FIVE_PERCENT)
              .setScale(2, RoundingMode.HALF_UP));

      // ** RULE: ALL CLAIMS WITH PENALTIES HAVE THEIR PPS TOTAL PENALTY
      // ** AMOUNT COMPUTED AS THE (PPS FEDERAL PENALTY AMOUNT PLUS THE
      // ** PPS LOW INCOME PATIENT PENALTY AMOUNT PLUS THE PPS OUTLIER
      // ** PENALTY AMOUNT PLUS THE PPS TEACH PENALTY AMOUNT.

      // COMPUTE PPS-TEACH-PAY-AMT =
      //            (PPS-TEACH-PAY-AMT - PPS-TEACH-PENALTY-AMT)
      prospectivePaymentRecord.setTeachPaymentAmount(
          prospectivePaymentRecord
              .getTeachPaymentAmount()
              .subtract(prospectivePaymentRecord.getTeachPenaltyAmount())
              .setScale(2, RoundingMode.HALF_UP));

      // COMPUTE PPS-TOTAL-PENALTY-AMT =
      //            (PPS-FED-PENALTY-AMT + PPS-LIP-PENALTY-AMT
      //            + PPS-OUT-PENALTY-AMT + PPS-TEACH-PENALTY-AMT).
      prospectivePaymentRecord.setTotalPenaltyAmount(
          prospectivePaymentRecord
              .getFederalPenaltyAmount()
              .add(prospectivePaymentRecord.getLowIncomePaymentPenaltyAmount())
              .add(prospectivePaymentRecord.getOutlierPenaltyAmount())
              .add(prospectivePaymentRecord.getTeachPenaltyAmount())
              .setScale(2, RoundingMode.HALF_UP));
    }
  }
}
