package gov.cms.fiss.pricers.irf.core.rules.rules2020;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import gov.cms.fiss.pricers.irf.core.ResultCode;
import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * Calculates the standard payment.
 *
 * <p>Converted from {@code 3500-CONTINUE-CALC} in the COBOL code.
 */
public class Irf2020CalculateStandardPayment
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public boolean shouldExecute(IrfPricerContext calculationContext) {
    return calculationContext.matchesReturnCode(ResultCode.OK_00);
  }

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // ****************************************************************
    // *  IF A TRANSFER CASE, CALCULATE THE STANDARD PAYMENT USING    *
    // *  THE PER DIEM IN THE CALCULATION.                            *
    // ****************************************************************
    if (calculationContext.isTransferCase()) {
      // COMPUTE PPS-STANDARD-PAY-AMT =
      //    (PPS-BDGT-NEUT-CONV-AMT / PPS-AVG-LOS)
      //    * (H-LOS + .5)
      //    * PPS-RELATIVE-WGT
      prospectivePaymentRecord.setStandardPaymentAmount(
          BigDecimalUtils.truncateDecimals(
              prospectivePaymentRecord
                  .getBudgetNeutralConversionAmount()
                  .divide(
                      BigDecimal.valueOf(prospectivePaymentRecord.getAverageLengthOfStay()),
                      4,
                      RoundingMode.DOWN)
                  .multiply(BigDecimal.valueOf(prospectivePaymentRecord.getLengthOfStay() + .5))
                  .multiply(prospectivePaymentRecord.getRelativeWeight())));
    } else {

      // ****************************************************************
      // *  IF A NON-TRANSFER CASE, CALCULATE THE STANDARD PAYMENT      *
      // *  AS DONE NORMALLY.                                           *
      // ****************************************************************

      // COMPUTE PPS-STANDARD-PAY-AMT =
      //      PPS-RELATIVE-WGT *
      //      PPS-BDGT-NEUT-CONV-AMT
      prospectivePaymentRecord.setStandardPaymentAmount(
          BigDecimalUtils.truncateDecimals(
              prospectivePaymentRecord
                  .getRelativeWeight()
                  .multiply(prospectivePaymentRecord.getBudgetNeutralConversionAmount())));
    }
  }
}
