package gov.cms.fiss.pricers.irf.core.tables;

import gov.cms.fiss.pricers.common.util.csv.CsvIngestionConfiguration;
import gov.cms.fiss.pricers.irf.IrfPricerConfiguration;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NavigableMap;
import java.util.concurrent.ConcurrentHashMap;

public class DataTables {
  private static final Map<Integer, DataTables> YEAR_LOOKUP = new ConcurrentHashMap<>();

  /** Corresponds to {@code CBSA-WI-TABLE} lookup table from the COBOL source. */
  private final Map<String, NavigableMap<LocalDate, CbsaTableEntry>> cbsaTable;
  /** Corresponds to {@code CMG-TABLE-DATA} lookup table from the COBOL source. */
  private final Map<String, CmgTableEntry> cmgTable;

  public DataTables(
      Map<String, NavigableMap<LocalDate, CbsaTableEntry>> cbsaTable,
      Map<String, CmgTableEntry> cmgTable) {
    this.cbsaTable = cbsaTable;
    this.cmgTable = cmgTable;
  }

  public static DataTables forYear(int pricerYear) {
    return YEAR_LOOKUP.get(pricerYear);
  }

  /**
   * Loads data tables.
   *
   * @param pricerConfiguration the pricer configuration to use
   */
  public static void loadDataTables(IrfPricerConfiguration pricerConfiguration) {
    final CsvIngestionConfiguration csvIngestionConfiguration =
        pricerConfiguration.getCsvIngestionConfiguration();
    final CbsaTableEntryLookupGenerator cbsaLookupProvider =
        new CbsaTableEntryLookupGenerator(csvIngestionConfiguration);
    final CmgTableEntryLookupGenerator cmgTableEntryLookupGenerator =
        new CmgTableEntryLookupGenerator(csvIngestionConfiguration);

    // Load all the tables
    for (final int pricerYear : pricerConfiguration.getSupportedYears()) {
      YEAR_LOOKUP.put(
          pricerYear,
          new DataTables(
              cbsaLookupProvider.generate(pricerYear),
              cmgTableEntryLookupGenerator.generate(pricerYear)));
    }
  }

  public CbsaTableEntry getCbsaWageIndex(String cbsa, String providedEffectiveDate) {
    final NavigableMap<LocalDate, CbsaTableEntry> entries = cbsaTable.get(cbsa);
    try {
      final LocalDate effectiveDate =
          DateTimeFormatter.BASIC_ISO_DATE.parse(providedEffectiveDate, LocalDate::from);

      if (entries != null) {
        final Entry<LocalDate, CbsaTableEntry> entry = entries.floorEntry(effectiveDate);
        return null == entry ? null : entry.getValue();
      }

      return null;
    } catch (final DateTimeParseException dtpe) {
      return null;
    }
  }

  public CmgTableEntry getCmgEntry(String cmgCode) {
    return this.cmgTable.get(cmgCode);
  }
}
