package gov.cms.fiss.pricers.irf.resources;

import com.codahale.metrics.annotation.Timed;
import gov.cms.fiss.pricers.common.api.InternalPricerException;
import gov.cms.fiss.pricers.common.api.YearNotImplementedException;
import gov.cms.fiss.pricers.common.application.errors.InputValidationError;
import gov.cms.fiss.pricers.common.application.errors.InternalPricerError;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.core.IrfPricerDispatch;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

@Consumes(MediaType.APPLICATION_JSON)
@Path("/price-claim")
@Produces(MediaType.APPLICATION_JSON)
public class IrfPricerResource {
  private final IrfPricerDispatch dispatch;

  public IrfPricerResource(IrfPricerDispatch dispatch) {
    this.dispatch = dispatch;
  }

  @POST
  @Timed
  @Operation(
      summary = "Prospective payment record for the given IRF bill",
      description =
          "Returns the Prospective Payment Record of the passed in IRF Claim. This record "
              + "includes details on how it was calculated as well as potential adjustments made.",
      responses = {
        @ApiResponse(
            responseCode = "200",
            description = "Success: Price of the passed in IRF claim",
            content = @Content(schema = @Schema(implementation = IrfOutput.class))),
        @ApiResponse(
            responseCode = "400",
            description =
                "Error: Bad Request.\n This status code covers requests that failed because of a problem with the syntax of the user's request:\n\n"
                    + "1) Request isn't valid JSON - The JSON body itself couldn't be parsed\n"),
        @ApiResponse(
            responseCode = "422",
            description =
                "Error: Year Not Implemented.\n A value of \"20110415\" was submitted for billingRecord.dischargeDate, but the Pricer API does not yet support this year.\n\n"
                    + "1) Request has a dischargeDate that is outside of the range of years covered by the pricer.\n\n"
                    + "Error: Unprocessable Entity.\n This status code covers requests that failed because of a problem with the values in the user's request:\n\n"
                    + "1) Request doesn't match schema - The request fails to pass a required field, passes the wrong datatype for a field, or passes an invalid enum value, or some other similar problem.\n"
                    + "2) Request is not semantically valid - The request contains conflicting values, for example if exactly one of fieldA or fieldB is accepted, but the caller passes values for both fields.\n"
                    + "3) Request values are out of bounds - The request passes a numeric value that is too large or too small, passes an array with too many elements, or a string that is empty or has invalid characters."
                    + "4) Request contains invalid dates - The request passes in date values that cannot be parsed correctly due to not being in the format of yyyyMMDD, was not a numerical value, or was greater than the allowed number of digits (8).",
            content = @Content(schema = @Schema(implementation = InputValidationError.class))),
        @ApiResponse(
            responseCode = "500",
            description =
                "Error: Internal pricer error. Please save the request data and send it to ffsm-team at ffsm@bellese.io if you see this message.",
            content = @Content(schema = @Schema(implementation = InternalPricerError.class)))
      })
  public IrfOutput price(
      @NotNull @Valid @Parameter(description = "IRF Claim to be priced", required = true)
          IrfInput input)
      throws YearNotImplementedException, InternalPricerException {
    return dispatch.process(input);
  }
}
